## Functions for paired design of count response (RNA-seq, ...)

####### Aggregating p-values of binomial tests 

#' Randomization null distributions for paired designs
#' @description 
#' Randomization null distributions for paired designs: 
#' \code{within-pair} where randomly flip the case-control asignment within
#' each pair; \code{between-pair} where the cases and controls remains as cases
#' or controls respectively but the correspondences between cases and controls
#' are randomly asigned. Finally, \code{complete} means that we select randomly 
#' the half of samples as cases and the rest as controls. The correspondence
#' case-control is randomly asigned too. 
#' @param y Design matrix (nrow(y) == ncol(x)). First column \code{y[,1]} 
#' is the block and second column \code{y[,2]} is the element within the block
#' with values 0 (control) and 1 (case) 
#' @param type Null randomization distribution:
#' "within-pair" where randomly flip the case-control asignment within
#' each pair; "between-pair" where the cases and controls remains as cases
#' or controls respectively but the correspondences between cases and controls
#' are randomly asigned. Finally, "complete" means that we select randomly the 
#' half of samples as cases and the rest as controls. The correspondence
#' case-control is randomly asigned too.
#' @author Guillermo Ayala 
#' @return 
#' Matrix with the blocks in the first column and the case-control asignment
#' in the second column  
#' @export
#' @seealso \link[OMICfpp]{pairedCounts}
null.paired = function(y,type = c("within-pair","between-pair","complete")){
  type = match.arg(type)
  x1 = y[,1]
  y1 = y[,2]
  if(type == "within-pair"){
    xf = factor(x1)
    tochange = which(stats::rbinom(nlevels(xf),size=1,prob=.5) == 1)
    y1[is.element(xf,levels(xf)[tochange]) & y[,2] == 1] = 0
    y1[is.element(xf,levels(xf)[tochange]) & y[,2] == 0] = 1
  }
  if(type == "between-pair")
    x1[which(y[,2] == 1)] = sample(x1[which(y[,2] == 1)])
  
  if(type == "complete"){
    n1 = length(x1)
    x1 = y1 = rep(0,n1)
    firstgroup = sample(n1,n1/2)
    secondgroup = setdiff(1:n1,firstgroup)
    x1[firstgroup] = 1:(n1/2)
    x1[secondgroup] = 1:(n1/2)
    y1[firstgroup] = 1
  }
  if(all(table(x1)!=2)) return("Output: Incomplete blocks")
  if(all(table(x1,y1)) != 1) return("Output: Incomplete pairs")
  cbind(x1,y1)
}

#' Differential expression for counts using ordered weighted averages
#' @description 
#' The data consists of case-control pairs.  The pair of counts per pair and
#' gene are compared by testing if the proportion of cases is equal to the 
#' proportion of the sample using the sizes of library
#' (\link[edgeR]{binomTest}).
#' It is assumed given (constant) the total count per pair and gene.
#' These p-values are aggregated using OWA for a given orness. The 
#' weights of the OWA are calculated using the probability function of 
#' a binomial distribution.   
#' @param x Matrix expression
#' @param y Design matrix (nrow(y) == ncol(x)). First column \code{y[,1]} 
#' is the block and second column \code{y[,2]} is the element within the block
#' with values 0 (control) and 1 (case) 
#' @param test Function to calculate the statistics per pairs
#' @param orness The orness for OWA aggregation
#' @param method Aggregation method
#' @param is.decreasing The values to aggregated should decreasing ordered
#' @author Guillermo Ayala and Alberto Berral 
#' @return 
#' A list with the aggregated p-values: result["dbinom"] and 
#' result["fuller"]
#' 
#' @export
#' @seealso \link[OMICfpp]{owa.weights}
pairedCounts = function(x,y,orness=.5,test,
                        method = c("dbinom","fuller"),
                        is.decreasing = FALSE){
  method = match.arg(method)
  txy = test(x,y)
  worness = sapply(orness,function(orness0)
    owa.weights(n=length(unique(y[,1])),orness=orness0,method=method))
  apply(worness,2,function(w) 
    apply(txy,1,function(x) sum(order(x,decreasing=is.decreasing) * w)))
}

#' Tests per row (feature) for marginal differential expression 
#' @description 
#' For a given expression matrix \code{x} and a design matrix
#' factor \code{y} the  \code{test} is applied. The expression
#' matrix probably is obtained as \code{x = exprs(gse)} or
#' \code{x = assay(se)} where \code{gse} is a \code{ExpressionSet} and 
#' \code{se} is a \code{RangedSummarizedExperiment}.
#' Differently to \link[tami]{rowtest} the output is a matrix where
#' each row vector corresponding to the differential expression of the 
#' corresponding row of the expression matrix.
#' @param x Matrix expression
#' @param y Design matrix (nrow(y) == ncol(x)). First column \code{y[,1]} 
#' is the block and second column \code{y[,2]} is the element within the block
#' with values 0 (control) and 1 (case) 
#' @param test Function with the statistical test (c(proportion.p,oddsratio.p,
#' oddsratio.stats))
#' @export
#' @return 
#' A data frame with columns named \code{statistic} (statistic of the test),
#' \code{rawp} (original p-values), \code{adjp} (adjusted p-values) and 
#' \code{qval} (q-values)
rowtest0 = function(x,y,test) tp = test(x,y) 

#' Testing equal proportions for comparing paired counts (RNA-seq,...) 
#' @description 
#' The data consists of case-control pairs.  The pair of counts per pair and
#' gene are compared by testing if the proportion of cases is equal to the 
#' proportion of the sample using the sizes of library (\link[edgeR]{binomTest}).
#' It is assumed given (constant) the total count per pair and gene.
#' The second choice is to use the odds-ratio (statistic or p-value)
#' 
#' @param x Count matrix
#' @param y Design matrix (nrow(y) == ncol(x)). First column \code{y[,1]} 
#' is the block and second column \code{y[,2]} is the element within the block
#' with values 0 (control) and 1 (case) 
#' @author Guillermo Ayala and Alberto Berral 
#' @return 
#' A \code{matrix} where each row corresponds with the different expression
#' statistics of the corresponding row of \code{x} for the common design given
#' by \code{y}
#' @export
proportion.p = function(x,y){
  y1 = factor(y[,1])
  y2 = as.logical(y[,2])
  sapply(levels(y1),function(i){
    sel = which(y1 == i)
    df0 = x[,sel]
    y0 = y2[sel]
    edgeR::binomTest(df0[,!y0],df0[,y0])
  })
}

#' statistic or p-value for odds-ratio equal 1 for comparing paired counts
#' (RNA-seq,...) 
#' @description 
#' The data consists of case-control pairs.  The pair of counts per pair and
#' gene are compared by testing if the proportion of cases is equal to the 
#' proportion of the sample using the sizes of library
#' (\link[edgeR]{binomTest}).
#' It is assumed given (constant) the total count per pair and gene.
#' The second choice is to use the odds-ratio (statistic or p-value)
#' 
#' @param x Count matrix
#' @param y Design matrix (nrow(y) == ncol(x)). First column \code{y[,1]} 
#' is the block and second column \code{y[,2]} is the element within the block
#' with values 0 (control) and 1 (case) 
#' @param type odds-ratio (stats) or p-value (testing if the odds-ratio
#' is equal to one)
#' @author Guillermo Ayala and Alberto Berral 
#' @return 
#' A \code{matrix} where each row corresponds with the different expression
#' statistics of the corresponding row of \code{x} for the common design given
#' by \code{y}
#' @export
oddsratio.paired = function(x,y,type = "p-value"){
  if(type == "p-value"){
    ## Defining fisherRow to extract p-value from fisher.test
    fisherRow = function(x0){
      temp = stats::fisher.test(matrix(x0, ncol = 2, byrow = FALSE))
      temp$p.value
    }
  }
  if(type == "stats"){
    ## Defining fisherRow to extract p-value from fisher.test
    fisherRow = function(x0){
      temp = stats::fisher.test(matrix(x0, ncol = 2, byrow = FALSE))
      temp$estimate
    }
  }
  y1 = factor(y[,1])
  y2 = as.logical(y[,2])
  sapply(levels(y1),function(i){
    sel = which(y1 == i)
    df0 = x[,sel]
    y0 = y2[sel]
    m1 = sum(df0[,!y0],na.rm=TRUE)
    m2 = sum(df0[,y0],na.rm=TRUE)
    countmatrix = cbind(df0[,!y0],m1-df0[,!y0],df0[,y0],m2-df0[,y0])
    t(apply(countmatrix, 1, fisherRow))
  })
}


#' p-value for odds-ratio equal 1 for comparing paired counts (RNA-seq,...) 
#' @description 
#' The data consists of case-control pairs.  The pair of counts per pair and
#' gene are compared by testing if the proportion of cases is equal to the 
#' proportion of the sample using the sizes of library (\link[edgeR]{binomTest}).
#' It is assumed given (constant) the total count per pair and gene.
#' The p-value for testing if the odds-ratio is equal to one is returned
#' 
#' @param x Count matrix
#' @param y Design matrix (nrow(y) == ncol(x)). First column \code{y[,1]} 
#' is the block and second column \code{y[,2]} is the element within the block
#' with values 0 (control) and 1 (case) 
#' @author Guillermo Ayala and Alberto Berral 
#' @return 
#' A \code{matrix} where each row corresponds with the different expression
#' statistics of the corresponding row of \code{x} for the common design given
#' by \code{y}
#' @export
oddsratio.p = function(x,y) oddsratio.paired(x,y,type="p-value")

#' stats for odds-ratio equal 1 for comparing paired counts (RNA-seq,...) 
#' @description 
#' The data consists of case-control pairs.  The pair of counts per pair and
#' gene are compared by testing if the proportion of cases is equal to the 
#' proportion of the sample using the sizes of library (\link[edgeR]{binomTest}).
#' It is assumed given (constant) the total count per pair and gene.
#' The odds-ratio is returned
#' 
#' @param x Count matrix
#' @param y Design matrix (nrow(y) == ncol(x)). First column \code{y[,1]} 
#' is the block and second column \code{y[,2]} is the element within the block
#' with values 0 (control) and 1 (case) 
#' @author Guillermo Ayala and Alberto Berral 
#' @return 
#' A \code{matrix} where each row corresponds with the different expression
#' statistics of the corresponding row of \code{x} for the common design given
#' by \code{y}
#' @export
oddsratio.stats = function(x,y) oddsratio.paired(x,y,type="stats")

 
#' Calculation of weights for OWA aggregation 
#' @description 
#' Calculation of weights for OWA aggregation. The method "dbinom" and
#' "fuller" are considered. 
#' @param n Number of values to aggregate
#' @param orness A value (vector) giving the orness to use 
#' (extreme values 0 and 1 are not allowed)
#' @param method The procedure used to produce the weigths
#' @export
#' @seealso \link[OMICfpp]{pairedCounts}
owa.weights = function(n,orness,method=c("dbinom","fuller")){
  if(method == "dbinom") w = dbinom(0:(n - 1), size = n - 1, prob = 1 - orness)
  if(method == "fuller") w = OMICfpp::fuller(orness = orness, n0 = n)
  w
}


#' @importFrom  stats dbinom uniroot 
NULL

#' OWA weigths using Fuller's method  
#' @description 
#' OWA weigths using Fuller's method
#' @param orness Orness 
#' @param n0 Number of features to aggregate 
#' @return 
#' Vector with the weigths for OWA 
#' @export
#' 
fuller = function(orness=0.5,n0=5){
  w = rep(0,n0)
  ffun= function(x,alpha=0.5,n=5){
    x *((n-1)*alpha + 1 - n* x)^n-((((n-1)*alpha)^(n-1)) * 
                                     (((n-1) *alpha - n )* x + 1))
  }
  wx = seq(0,1,len=100)
  ##plot(wx,ffun(wx),type="l")
  w[1] = uniroot(ffun,interval=c(0.01,.99),tol=0.00001,alpha=orness,n=n0)$root
  w[n0] = (((n0-1)*orness -n0)*w[1] + 1) / ((n0-1) * orness + 1 - n0 * w[1])
  for(j in 2:(n0-1))
    w[j] = (w[1]^(n0-j) * w[n0]^(j-1))^(1/(n0-1))
  w = w / sum(w)
  w
}

#' @importFrom  stats p.adjust
NULL

#' Analyzing the p-values
#' @description
#' Different p-values are produced for each gene set using a different
#' statistical procedure and a randomization distribution. The p-values
#' obtained are used to test the self-contained hypothesis of gene
#' set diffferential expression
#' @param pval List where each element contains the p-values for each test
#' and randomization distribution
#' @param oneRealization One single sample to be used?
#' @param aggr Function to aggregate the p-values (as proposed in
#' \link{metap})
#' @param correction Function to adjust the p-values of the meta-analysis
#' of p-values (single = FALSE) or the raw p-values (single = TRUE)
#' @param  gsc Gene set collection 
#' @param foutput Output file
#' @export
doIt = function(pval,oneRealization = FALSE,aggr=NULL,correction ="BH",
                gsc,foutput="output"){
    if(!oneRealization){
        p = sapply(pval,function(x) apply(x+.0000001,1,function(x) aggr(x)$p))
    }
    else{ 
        p = matrix(NA,ncol=length(pval),nrow=length(gsc))
        for(i in 1:length(pval)) p[,i] = pval[[i]]        
    }

    adjp  = apply(p,2,p.adjust,method= correction)
    GO = tami::go2url(names(gsc))
    df = data.frame(GO)
    names0 = "GO"
    for(i in 1:length(pval)){
        names0 = c(names0,names(pval)[i],paste0("adj ",names(pval)[i]))
        df = data.frame(df,p[,i],adjp[,i],stringsAsFactors=FALSE)
    }
    names(df) = names0
    htmlRep1 = ReportingTools::HTMLReport(shortName = foutput,title = foutput,
                          reportDirectory = "./reports")
    ReportingTools::publish(df,htmlRep1)
    ReportingTools::finish(htmlRep1)
    df
}

#' Generation of randomization p-values
#' @description
#' Different p-values are produced for each gene set using a different
#' statistical procedure and a randomization distribution. The p-values
#' obtained are used to test the self-contained hypothesis of gene
#' set diffferential expression
#' @param df Count matrix
#' @param y Vector giving the pairs of columns
#' @param test Function comparing counts within each pair
#' @param orness Orness 
#' @param nsim Number of simulations
#' @param method Method to generate the orness values. See \link{owa.weights}.
#' @param type Randomization distribution to generate random pairs
#' @param is.decreasing  The values to aggregated should decreasing ordered
#' @param to.save Is to be saved in \code{foutput}?
#' @param foutput Output file
#' 
#' @export

marginal.analysis = function(df,y,test,orness,nsim = 100,
                             method="dbinom",type="within-pair",
                             is.decreasing=FALSE,to.save = FALSE,
                             foutput="marginal.analysis.rda"){
    df1 = matrix(NA,nrow = nrow(df),ncol=length(orness)*(nsim+1))
    rownames(df1) = rownames(df)
     cat("Permutation 0 \n")
    df1[,1:length(orness)] =
        pairedCounts(x=df,y=y,test,orness=orness, method = method,
                     is.decreasing=is.decreasing)
    for (j in 1:nsim){
        cat("Permutation ",j,"\n")
        df1[,j * length(orness) + 1:length(orness)] = 
            pairedCounts(x=df,y=null.paired(y,type),test=test,orness=orness,
                         method=method,is.decreasing=is.decreasing)
    }
    if(to.save) save(df1,file=foutput)
    df1
}

#' Calculation of the p-values for different ornesses
#' @description
#' Different p-values are produced for each gene using 
#' randomization distribution.
#' @param x Count matrix
#' @param y Vector giving the pairs of columns and the treatment (first and
#' second column)
#' @param test Function comparing counts within each pair
#' @param orness Vector of the ornesses
#' @param  nsim Number of simulations 
#' @param method Method to generate the orness values
#' \code{c("dbinom","fuller")}. See \link{owa.weights}.
#' @param type Randomization distribution to generate random pairs
#' @param is.decreasing  The values to be  aggregated in decreasing order
#' @export
p.orness = function(x,y,test,orness,nsim = 100,method="dbinom",
                    type="between-pair",is.decreasing=FALSE){
  
  df1 = marginal.analysis(df=x,y=y,test=test,
                    orness= orness,nsim = nsim,method=method,
                    type=type,is.decreasing=is.decreasing)
  pval0 = df1[,1:length(orness)]
  pval1 = matrix(0,nrow=nrow(df1),ncol=length(orness))
  for(i in 1:nsim)
    pval1 = pval1 + (df1[,(1:length(orness))+ i*length(orness)] < pval0)*1
  p.value = pval1/nsim
  p.value
}

#' Choosing an orness
#' @description
#' The p-value matrix is evaluated for different orness where each orness 
#' correspond with a different column in the matrix. 
#' @param pval Matrix of p-values
#' @param orness Vector with the ornesses 
#' @param nmin Minimum of n to evaluate
#' @param nmax Maximum of n to evaluate
#' @export
#'
chooseOrness = function(pval,orness,nmin=5,nmax=50){
    best.o = NULL
    for(n0 in nmin:nmax){
        u0 = apply(pval,2,diffMeanTail,n=n0)
        best.o = c(best.o,orness[which.min(u0)])
    }
    best.o[which.min(abs(best.o-.5))]
}

#' Difference between the means in the tails per column
#' @description
#' For a matrix \code{x}, the difference between the mean of the
#' n lowest values and the mean of the n greatest values is calculated
#' It is an auxiliary function for \code{chooseOrness}
#' @param x Matrix
#' @param n Number of values in each tail
#' @export
diffMeanTail = function(x,n){
    orden = sort(x,index.return=TRUE)$ix
    ff = rep(1:0,each=n)
    lower.tail = orden[1:n] 
    upper.tail = orden[(length(x)-n+1):length(x)]
      mean(x[lower.tail])  - mean(x[upper.tail])
}


#' p-values and orness using the owa-randomization procedure
#' @description
#' Different p-values are produced for each gene using 
#' randomization distribution. 
#' @param x Count matrix
#' @param y Vector giving the pairs of columns and the treatment (first and
#' second column)
#' @param test Function comparing counts within each pair
#' @param orness Vector of the ornesses
#' @param norness Number of values for orness if orness == NULL
#' @param  nsim Number of simulations 
#' @param method Method to generate the orness values
#' \code{c("dbinom","fuller")}. See \link{owa.weights}.
#' @param type Randomization distribution to generate random pairs
#' @param is.decreasing  The values to be  aggregated in decreasing order
#'
#' @export
#' 
omicf = function(x,y,test=proportion.p,orness=NULL,norness=10,nsim = 100,
                 method="dbinom",type="complete",is.decreasing=FALSE){
    if(length(orness) == 1 &  !is.null(orness)){
        p.value = p.orness(x=x,y=y,test=test,orness =orness,nsim = nsim,
                           method=method,type=type,is.decreasing=is.decreasing)
        p.value = as.vector(p.value)
        return(list(p.value = p.value, orness=orness))
    }
    
    if(length(orness) > 1){
        p.temp = p.orness(x=x,y=y,test=test,orness =orness,nsim = 10,
                          method=method,type=type,is.decreasing=is.decreasing)
        orness = chooseOrness(p.temp,orness=orness,nmin=5,nmax=20)
        p.value = p.orness(x=x,y=y,test=test,orness =orness,nsim = nsim,
                           method=method,type=type,is.decreasing=is.decreasing)
        p.value = as.vector(p.value)
        return(list(p.value = p.value,orness=orness))
    }
    
    if(is.null(orness)){
        orness = seq(0.05,0.95,length.out = norness)
        p.temp = p.orness(x=x,y=y,test=test,orness =orness,nsim = 10,
                          method=method,type=type, is.decreasing=is.decreasing)
        orness = chooseOrness(p.temp,orness=orness,nmin=5,nmax=20)
        p.value = p.orness(x=x,y=y,test=test,orness =orness,nsim = nsim,
                           method=method,type=type, is.decreasing=is.decreasing)
        p.value = as.vector(p.value)
        return(list(p.value = p.value,orness=orness))
    }
}


#' @importFrom  edgeR  DGEList estimateDisp glmQLFit glmQLFTest topTags
NULL

#' edgeR with paired design
#' @description
#' Method implemented in \link{edgeR} for paired design of RNA-Seq
#' data
#' @param x Expression matrix
#' @param y Design matrix (nrow(y) == ncol(x)). First column \code{y[,1]} 
#' is the block and second column \code{y[,2]} is the element within the block
#' with values 0 (control) and 1 (case) 
#' @export

edgeRpaired = function(x,y){
    subject = factor(y[,1])
    treatment = factor(y[,2])
    x1 = edgeR::DGEList(x)
    design = model.matrix(~ subject + treatment)
    x1 = edgeR::estimateDisp(x1,design)
    fit =  edgeR::glmQLFit(x1, design)
    qlf =  edgeR::glmQLFTest(fit)
    qlf1 = edgeR::topTags(qlf,sort.by="none",n=nrow(x1))
    qlf1$table[,"PValue"]
}

#' @importFrom  S4Vectors  DataFrame
NULL

#' @importFrom  DESeq2  DESeq lfcShrink DESeqDataSetFromMatrix
NULL

#' @importFrom  stats  model.matrix
NULL

#' DESeq2 with paired design
#' @description
#' Method implemented in DESeq2 for paired design of RNA-Seq
#' data. The p-values are returned.
#' @param x Expression matrix
#' @param y Design matrix (nrow(y) == ncol(x)). First column \code{y[,1]} 
#' is the block and second column \code{y[,2]} is the element within the block
#' with values 0 (control) and 1 (case) 
#' @export
#' 
DESeq2paired = function(x,y){
    y1 = S4Vectors::DataFrame(subject = factor(y[,1]),treatment=factor(y[,2]))
    dds <- DESeq2::DESeqDataSetFromMatrix(countData = x,
                                          colData = y1,
                                          design = ~ subject + treatment)
    dds = DESeq2::DESeq(dds)
    resLFC3= DESeq2::lfcShrink(dds, coef=nlevels(factor(y[,1]))+1)
    resLFC3[,"pvalue"]
}


#' @importFrom  ROC rocdemo.sca AUC
NULL

#' Function to simulation study
#' @description
#' Function to simulation study
#' @param n Number of pairs
#' @param  N Number of genes
#' @param  nsig Number of significant genes (rows (noutlier+1):(noutlier+nsig)
#' @param noutlier Vector giving the number of outlier genes noutlier[1]
#' (rows 1:noutlier[1] in the expression matrix) and number of pairs
#' (columns 1:noutlier[2])
#' @param lambda Vector with the lambda values for  tami::rPairedPoisson
#' @param test Function comparing counts within each pair
#' @param norness Number of values for orness if orness == NULL
#' @param  nsim Number of simulations 
#' @param method Method to generate the orness values
#' \code{c("dbinom","fuller")}. See \link{owa.weights}.
#' @param type Randomization distribution to generate random pairs
#' @param is.decreasing  The values to be  aggregated in decreasing order
#' @param proc Procedures to be tested: \code{owb} OWA with between-pair
#' distribution, \code{owc} OWA with complete distribution, \code{er}
#' edgeR, \code{de} DESEq2.
#' @param do.auc Is the AUC value to be calculated?
#' @export
#' @keywords Simulation study
#' 
justDo = function(n,N,nsig,noutlier=c(1,1),lambda,
                  test=proportion.p,norness=50,nsim=100,
                  method="dbinom",type="complete",is.decreasing=FALSE,
                  proc = c("owb","owc","er","de"),do.auc = FALSE){
    y = cbind(rep(1:n,each=2),rep(c(0,1),n))
    outlier = NULL
    for(j  in 1:noutlier[2]){
        outlier.temp = cbind(1:noutlier[1],rep(j,noutlier[1]))
        outlier = cbind(outlier,outlier.temp)
        }
    sig = (noutlier[1]+1):(noutlier[1]+nsig)
    if(do.auc){
        fsig = rep(0,N)
        fsig[sig]=1
        }
    auc.all = NULL
    p.value = NULL
    
    x = tami::rPairedPoisson(n=n,N=N,lambda1=lambda[1],lambda2=lambda[2],
                              lambda3=lambda[3],lambda4=lambda[4],
                              gs=sig,outlier=outlier)
    
    if(is.element("owb",proc)){
        p.ow = omicf(x=x,y=y,test=test,norness=norness,nsim = nsim,
                     method="dbinom",type="between-pair",
                         is.decreasing=FALSE)
        p.owb = p.ow$p.value
        p.value = cbind(p.value,p.owb)
        if(do.auc){
            p.owb.r = ROC::rocdemo.sca(truth= fsig,data=1-p.owb)
            auc.all = c(auc.all,ROC::AUC(p.owb.r))
            }
    }
    if(is.element("owc",proc)){
        p.ow = omicf(x=x,y=y,test= test,norness=norness,nsim = nsim,
                     method="dbinom",type="complete",is.decreasing=FALSE)
        p.owc = p.ow$p.value
        p.value = cbind(p.value,p.owc)
        if(do.auc){
            p.owc.r = ROC::rocdemo.sca(truth= fsig,data=1-p.owc)
            auc.all = c(auc.all,ROC::AUC(p.owc.r))
            }
    }
    
    if(is.element("er",proc)){
        p.er = edgeRpaired(x=x,y=y)
        p.value = cbind(p.value,p.er)
        if(do.auc){
            p.er.r = ROC::rocdemo.sca(truth= fsig,data=1-p.er)
            auc.all = c(auc.all,ROC::AUC(p.er.r))
            }
    }
    if(is.element("de",proc)){
        p.de = DESeq2paired(x=x,y=y)
        p.value = cbind(p.value,p.de)
        if(do.auc){
            p.de.r = ROC::rocdemo.sca(truth= fsig,data=1-p.de)
            auc.all = c(auc.all,ROC::AUC(p.de.r))
            }
    }
    if(!do.auc) auc.all = NULL
    list(p.value = p.value,auc = auc.all)
}


#' Function to simulation study
#' @description
#' Function to simulation study
#' @param n Number of pairs
#' @param  N Number of genes
#' @param  nsig Number of significant genes (rows (noutlier+1):(noutlier+nsig)
#' @param noutlier Vector giving the number of outlier genes noutlier[1]
#' (rows 1:noutlier[1] in the expression matrix) and number of pairs
#' (columns 1:noutlier[2])
#' @param theta Vector with the lambda values for  tami::rPairedPoisson
#'  or matrix with two columns and 4 rows where the entry theta[i,1]
#' is the mean and 1/theta[i,2] the dispersion for the i-th random variable
#' @param model Distribution of the random counts: "Poisson" or "NB"
#' for Poisson or negative binomial distribution
#' @param test Function comparing counts within each pair
#' @param norness Number of values for orness if orness == NULL
#' @param  nsim Number of simulations 
#' @param method Method to generate the orness values
#' \code{c("dbinom","fuller")}. See \link{owa.weights}.
#' @param is.decreasing  The values to be  aggregated in decreasing order
#' @param proc Procedures to be tested: \code{owb} OWA with between-pair
#' distribution, \code{owc} OWA with complete distribution, \code{er}
#' edgeR, \code{de} DESEq2.
#' @param do.auc Is the AUC value to be calculated?
#' @export
#' @keywords Simulation study
#' 
justDo2 = function(n,N,nsig,noutlier=c(1,1),theta,model=c("Poisson","NB"),
                  test=proportion.p,norness=50,nsim=100,
                  method="dbinom",is.decreasing=FALSE,
                  proc = c("owb","owc","er","de"),do.auc = FALSE){
    y = cbind(rep(1:n,each=2),rep(c(0,1),n))
    outlier = NULL
    for(j  in 1:noutlier[2]){
        outlier.temp = cbind(1:noutlier[1],rep(j,noutlier[1]))
        outlier = cbind(outlier,outlier.temp)
        }
    sig = (noutlier[1]+1):(noutlier[1]+nsig)
    if(do.auc){
        fsig = rep(0,N)
        fsig[sig]=1
        }
    auc.all = NULL
    p.value = NULL

    if(model == "Poisson")
        x = rPairedPoisson(n=n,N=N,lambda1=theta[1],lambda2=theta[2],
                                 lambda3=theta[3],lambda4=theta[4],
                                 gs=sig,outlier=outlier)
    if(model == "NB")
        x = rPairedNB(n=n,N=N,
                          mu1=theta[1,1],size1=theta[1,2],
                          mu2=theta[2,1],size2=theta[2,2],
                          mu3=theta[3,1],size3=theta[3,2],
                          mu4=theta[4,1],size4=theta[4,2],
                          gs = sig,outlier=outlier)
    
    if(is.element("owb",proc)){
        p.ow = omicf(x=x,y=y,test=test,norness=norness,nsim = nsim,
                     method="dbinom",type="between-pair",
                         is.decreasing=FALSE)
        p.owb = p.ow$p.value
        p.value = cbind(p.value,p.owb)
        if(do.auc){
            p.owb.r = ROC::rocdemo.sca(truth= fsig,data=1-p.owb)
            auc.all = c(auc.all,ROC::AUC(p.owb.r))
            }
    }
    if(is.element("owc",proc)){
        p.ow = omicf(x=x,y=y,test= test,norness=norness,nsim = nsim,
                     method="dbinom",type="complete",is.decreasing=FALSE)
        p.owc = p.ow$p.value
        p.value = cbind(p.value,p.owc)
        if(do.auc){
            p.owc.r = ROC::rocdemo.sca(truth= fsig,data=1-p.owc)
            auc.all = c(auc.all,ROC::AUC(p.owc.r))
            }
    }
    
    if(is.element("er",proc)){
        p.er = edgeRpaired(x=x,y=y)
        p.value = cbind(p.value,p.er)
        if(do.auc){
            p.er.r = ROC::rocdemo.sca(truth= fsig,data=1-p.er)
            auc.all = c(auc.all,ROC::AUC(p.er.r))
            }
    }
    if(is.element("de",proc)){
        p.de = DESeq2paired(x=x,y=y)
        p.value = cbind(p.value,p.de)
        if(do.auc){
            p.de.r = ROC::rocdemo.sca(truth= fsig,data=1-p.de)
            auc.all = c(auc.all,ROC::AUC(p.de.r))
            }
    }
    list(p.value = p.value,auc = auc.all)
}

#' Random paired counts using Poisson distributions  
#' @description 
#' A gene expression matrix of counts with \code{N} features and
#' \code{length(n)} pairs of samples.
#' The first pair corresponds to the two first column and so on.
#' The total number of samples is  \code{2*n}
#' The distribution of  the first component is Poisson with
#' parameter \code{lambda1}. For the genes (rows) in
#' \code{gs} the distribution is the sum of the first generated value
#' plus a random count with Poisson distribution with mean \code{lambda2}.
#' For the genes (rows) not in  \code{gs} the distribution of the second component
#' is the sum of  the first generated value 
#' plus a random count with Poisson distribution with mean \code{lambda3}.
#' The matrix \code{outlier} contains in the first column the indices of rows and in
#' the second column the indices of pairs. For the pairs of counts in \code{outlier}
#' the first component follows a Poisson distribution with mean \code{lambda1}
#' and the second component is equal to the first one plus a random count with
#' Poisson distributed with mean \code{lambda4}.
#' 
#' @param n Number of pairs
#' @param  N Number of genes
#' @param lambda1 Mean of the first component
#' @param lambda2 Mean of the random count added to the first generated value
#' on the significant genes
#' @param lambda3 Mean of the random count added to the first generated value
#' on the non significant genes
#' @param lambda4 Mean of the random count added to the first generated value
#' on the outlier genes given in \code{outlier}
#' @param gs Significant gene set (a subset of rows in 1:N)
#' @param outlier Matrix (rows as genes and pairs as samples) where
#' the corresponding pair of counts are an outlier
#' gs == NULL correspond with no differential expression
#' 
#' @export
#' @keywords 
#' Random counts 
#'
rPairedPoisson = function(n,N,lambda1,lambda2,lambda3,lambda4,
                          gs = NULL,outlier=NULL){
    
    x0 = matrix(NA,nrow=N,ncol=2*n)
    group1 = seq(1,2*n-1,2)
    group2 = seq(2,2*n,2)
    x0[,group1] = matrix(stats::rpois(N*n,lambda=lambda1),nrow=N,ncol=n)
    if(!is.null(gs))
        x0[gs,group2] = matrix(stats::rpois(n*length(gs),lambda=lambda2),
                               nrow=length(gs),ncol=n)
    if(!is.null(gs) & length(gs) < N)
        x0[-gs,group2] = matrix(stats::rpois((N-length(gs))*n,lambda=lambda3),
                                nrow=N-length(gs),ncol=n)
    if(is.null(gs))
        x0[,group2] = matrix(stats::rpois(N*n,lambda=lambda3),nrow=N,ncol=n)
    if(!is.null(outlier)){
        outlier= as.matrix(outlier)
        x0[outlier[,1],2*outlier[,2]] =
            matrix(stats::rpois(nrow(outlier)*ncol(outlier),lambda=lambda4),
                   nrow=nrow(outlier),ncol=nrow(outlier))
    }

    x0[,group2] = x0[,group1] + x0[,group2]
    x0   
}


#' Random paired counts using independent negative binomial distributions  
#' @description 
#' A gene expression matrix of counts with \code{N} features and
#' \code{length(n)} pairs of samples.
#' The first pair corresponds to the two first column and so on.
#' The total number of samples is  \code{2*n}
#' The distribution of  the first component (control) is negative binomial
#' with  mean \code{mu1} and dispersion parameter \code{1/size1} (the variance
#' is equal to \code{mu1 + (mu1^2)/size1}.
#' For the genes (rows) in \code{gs} the distribution of the second
#' component of the pair (case) is a negative binomial distribution
#' with mean \code{mu2} and dispersion 1/size2.
#' For the genes (rows) not in  \code{gs} (significant genes) the
#' distribution of the second component is negative binomial  with mean
#' \code{mu3} and dispersion 1/size3.
#' The matrix \code{outlier} contains in the first column the indices of
#' rows and in #' the second column the indices of pairs.
#' For the pairs of counts in \code{outlier}
#' the first component follows a negative binomial  distribution with mean
#' \code{mu1} and dispersion 1/size1 and the second component
#' follows a negative binomial distribution with mean
#' \code{mu4} and dispersion 1/size4.
#' 
#' @param n Number of pairs
#' @param  N Number of genes
#' @param  mu1 Mean of the first component
#' @param  size1  1/size1 is the dispersion  of the first component
#' @param mu2 Mean of the random count added to the first generated value
#' on the significant genes
#' @param size2   1/size2 is the dispersion of the random count added to the
#' first generated value on the significant genes
#' @param mu3 Mean of the random count added to the first generated value
#' on the non significant genes
#' @param size3 1/size3   is the dispersion of the random count added to the
#' first generated value on the non significant genes
#' @param mu4 Mean of the random count added to the first generated value
#' on the outlier genes given in \code{outlier}
#' @param size4 1/size4   is the dispersion of the random count added to the
#' first generated value on the outlier genes
#' @param gs Significant gene set (a subset of rows in 1:N)
#' @param outlier Matrix (rows as genes and pairs as samples) where
#' the corresponding pair of counts are an outlier
#' gs == NULL correspond with no differential expression
#' 
#' @export
#' @keywords 
#' Random counts 
#'
rPairedNB = function(n,N,mu1,size1,mu2,size2,mu3,size3,mu4,size4,
                          gs = NULL,outlier=NULL){
    
    x0 = matrix(NA,nrow=N,ncol=2*n)
    group1 = seq(1,2*n-1,2)
    group2 = seq(2,2*n,2)
    ## Controls
    x0[,group1] = matrix(stats::rnbinom(N*n, size=size1,mu=mu1),nrow=N,ncol=n)
    ## Significant genes 
    if(!is.null(gs))
        x0[gs,group2] = matrix(stats::rnbinom(n*length(gs),size=size2,mu=mu2),
                               nrow=length(gs),ncol=n)
    if(!is.null(gs) & length(gs) < N)
        x0[-gs,group2] =
            matrix(stats::rnbinom((N-length(gs))*n,size=size3,mu=mu3),
                   nrow=N-length(gs),ncol=n)
     if(is.null(gs))
     x0[,group2] = matrix(stats::rnbinom(N*n, size=size3,mu=mu3),nrow=N,ncol=n)
    
    if(!is.null(outlier)){
        outlier= as.matrix(outlier)
        x0[outlier[,1],2*outlier[,2]] =
            matrix(stats::rnbinom(nrow(outlier)*ncol(outlier),
                                  size=size4,mu=mu4),
                   nrow=nrow(outlier),ncol=nrow(outlier))
    }
    x0[,group2] = x0[,group1] + x0[,group2]
    x0   
}

###############################################
############# Functions for specific plots used in the paper ########
###############################################

#' @importFrom  ggplot2 ggplot aes geom_density xlim stat_ecdf geom_line ylab
NULL

#' @importFrom  ggplot2 ylim geom_point facet_wrap geom_smooth geom_hline
NULL

#' @importFrom  gridExtra grid.arrange
NULL

#' Plots the original p-values of within pairs comparison
#' @description
#' Using the within pair comparison p-values, the between-pair and the complete
#' randomization p-values, the density and empirical cumulative distribution
#' function of the original p-values. The between-pair and complete p-values
#' are plotted against the corresponding orness.
#' @param p0 Original within pair p-values
#' @param p.b Between-pair randomization p-values
#' @param p.c Complete randomization p-values
#' @param orness The orness for OWA aggregation
#' @param row The row in the expression matrix of the gene
#' @param id Gene identifier in \code{rownames(p0)}
#'
#' @export
evalOrness = function(p0,p.b,p.c,orness,row=NULL,id=NULL){
    if(!is.null(id)) row = which(rownames(p0) == id)
    if(!is.null(row)){
        df = data.frame(p0 = p0[row,])
        plot1 = ggplot2::ggplot(df,ggplot2::aes(x=p0)) + ggplot2::geom_density() +
            ggplot2::xlim(0,1)
        plot2 = ggplot2::ggplot(df,ggplot2::aes(x=p0)) + ggplot2::stat_ecdf()+
            ggplot2::xlim(0,1)
        df = data.frame(p.b = p.b[row,],orness=orness)
        plot3 = ggplot2::ggplot(df,ggplot2::aes(x=orness,y=p.b)) +
            ggplot2::geom_line() + ggplot2::ylim(0,1)
        df = data.frame(p.c = p.c[row,],orness=orness)
        plot4 = ggplot2::ggplot(df,ggplot2::aes(x=orness,y=p.c)) +
            ggplot2::geom_line() + ggplot2::ylim(0,1)
        return(gridExtra::grid.arrange(plot1, plot2,plot3,plot4, ncol = 2,nrow=2))
    }
    if(is.null(row) & is.null(id)) return("Row or identifier needed")
}

#' Comparative plot of the randomization p-values 
#' @description
#' The randomization p-values are plotted against the corresponding orness.
#' @param genes Identifiers of the genes
#' @param rowgenes Rows in the expression matrix corresponding to \code{genes}
#' @param p1 Randomization p-values
#' @param orness The orness for OWA aggregation
#' @export
comp.profiles = function(genes,rowgenes,p1,orness){
    p3 = data.frame(t(p1[rowgenes,]),orness = orness)
    names(p3) = c(as.character(genes),"orness")
    aa = reshape2::melt(p3[,-ncol(p3)],id=orness)
    df0 = data.frame(aa,orness1=rep(orness,length(genes)))
    p = ggplot2::ggplot(df0,ggplot2::aes(x=orness1,y=value)) +
        ggplot2::geom_point() 
    p = p + ggplot2::facet_wrap(~variable,labeller=label_value)+
        ggplot2::geom_smooth(span=.2) 
    p + ggplot2::geom_hline(yintercept = .001) + ggplot2::ylab("")
}
