
#' @importFrom  stats ecdf
NULL

#' MonteCarlo p-value for Random labelling hypothesis for gene point process
#' @description
#' MonteCarlo p-value for Random labelling hypothesis for gene point process
#' \link[genefilter]{rowttests}
#' @param GeneSet Gene set (given as rows in the expression matrix
#' \code{exprm}
#' @param exprm Expression matrix
#' @param cases Cases (given as columns in the expression matrix \code{exprm})
#' @param controls Controls (given as columns in the expression matrix \code{exprm})
#' @param test Test
#' @param GeneNullDistr Gene null distribution 
#' @param nmax Number of replications 
#' @return  p-value
#' @export
#' 
RandomLabelling = function(GeneSet,exprm,cases,controls,test,
                           GeneNullDistr="randomization",nmax=100){
    exprm0 = exprm[,c(cases,controls)]
    n1 = length(cases);n2=length(controls); n = n1+n2
    if(GeneNullDistr == "randomization"){
        if(choose(n,n1) <= nmax){
            comb0  = combinat::combn(n,n1)
            nmax = choose(n,n1)
            }else{
            comb0 = matrix(NA,nrow=n1,ncol=nmax)
            comb0[,1] = 1:n1
            for(i in 2:nmax) comb0[,i] = sample(n,n1)
        }
        comb1 = rbind(comb0,apply(comb0,2,setdiff,x=1:n))
    }
    if(GeneNullDistr == "bootstrap"){  
        comb1 = matrix(NA,nrow=n,ncol=nmax)
        comb1[,1] = 1:n
        for(i in 2:nmax) comb1[,i] = sample(n,replace = TRUE)
    }
    
    t0 = sapply(1:ncol(comb1),function(i){
        labels=c(rep(1,n2),rep(0,n1))
        x=exprm0[GeneSet,comb1[,i]]
        y=t(rbind(x,labels))
        row.names(y)=c(1:n)
        x1 = spatstat::ppx(y,coord.type=c(rep('s',nrow(x)),'m') )         
        neighbors = spatstat::nnwhich(x1)
        sum(labels*labels[neighbors])
    })
  sum(t0[-1]<=t0[1])/nmax
}


#' @importFrom  spatstat ppx nnwhich
NULL

#' Cuzzick and Edward test
#' @description 
#' Cuzzick and Edward statistic for point pattern of p-values as case process
#' and point pattern of p-values using the \code{within-pair} randomization
#' as control process
#' @param GeneSet Rows corresponding to gene set
#' @param exprm Matrix with the p-values
#' @param cases Columns of the original data
#' @param controls Columns of the control data
#' @return 
#' Percentile of the real statistic with respect to the randomization
#' values
#' @export
#' 
CuzickEdwards = function(GeneSet,exprm,cases,controls){
    labels=c(rep(1,length(controls)),rep(0,length(cases)))
    x=exprm[unlist(GeneSet),c(cases,controls)]
    p=dim(x)[1]
    y=t(rbind(x,labels))
    row.names(y)=c(1:24)
    X = spatstat::ppx(y,coord.type=c(rep('s',p),'m') )         
    vecinos = spatstat::nnwhich(X)
    sum(labels*labels[vecinos])
}

#' Diggle, Morris and Morton-Jones test
#' @description 
#' Diggle, Morris and Morton-Jones statistic for point pattern of p-values
#' as case process and point pattern of p-values using the \code{between-pair}
#' randomization as control process
#' @param GeneSet Rows corresponding to gene set
#' @param exprm Matrix with the p-values
#' @param cases Columns of the original data
#' @param controls Columns of the control data
#' @return 
#' Percentile of the real statistic with respect to the randomization
#' values
#' @export
#' 
DiggleMorrisMorton = function(GeneSet,exprm,cases,controls){
    dist0=apply(exprm[unlist(GeneSet),controls],2,function(x) t(x)%*%x)
    dist1=apply(exprm[unlist(GeneSet),cases],2,function(x) t(x)%*%x)
    dist=c(dist0,dist1)
    n0=length(dist0)
    n1=length(dist1)
    n=n0+n1
    deltas=c(rep(1,n0),rep(0,n1))
    k=round(n*0.1)
    ordenadas=deltas[order(dist)]
    sumascorregidas=matrix(rep(NA,n*n),ncol=n)
    for (s in 1:n-1) {
        for (t in (s+1):n) {
            sumascorregidas[s,t]=sum(ordenadas[s:t])/(t-s+1)
        }
    }
    sumascorregidas[1,1]=ordenadas[1]
    sumascorregidas[n,n]=ordenadas[n]
    thetas=c()
    for (i in 1:(n-1)){
        maximos=apply(sumascorregidas[,i:n],1,max,na.rm = TRUE)
        thetas[i]=min(maximos[1:i])
    }
    thetas[n]=sumascorregidas[n,n]
    2*sum(log(thetas[ordenadas>0]/(n0/n)))+
        2*sum(log((1-thetas[ordenadas<1])/(n1/n)))
}

#' @importFrom  stats optim pchisq
NULL

############################################
#############  Diggle90 ####################
############################################

#' diggle90
#' Method proposed in Diggle (1990)
#' @description
#' Peter J. Diggle,
#' A Point Process Modelling Approach to Raised Incidence of a Rare Phenomenon
#' in the Vicinity of a Prespecified Point,
#' Journal of the Royal Statistical Society. Series A (Statistics in Society)
#' 1990. 153 (3) 349-362.
#' @param GeneSet Gene set
#' @param exprm Expression matrix
#' @param cases Columns for cases in expression matrix
#' @param controls Columns for controls in expression matrix
#' @return
#' p-value
#' @export
#' 
diggle90 = function(GeneSet,exprm,cases,controls){
    dist0=apply(exprm[unlist(GeneSet),controls],2,function(x) t(x)%*%x)
    dist1=apply(exprm[unlist(GeneSet),cases],2,function(x) t(x)%*%x)
    n0=length(dist0)
    n1=length(dist1)
    h=0.1
    alpha0=0.1
    beta0=0.1
    parmax=optim(c(alpha0,beta0),.Lneg,.Lgrneg,dist0,dist1,h,method="BFGS")
    alphamax=parmax$par[1]
    betamax=parmax$par[2]
    statistic = 2*(-parmax$value+.Lneg(c(0,0),dist0,dist1,h))
    p.value = 1-pchisq(statistic,df=2)
    p.value
}

#' Auxiliary function for diggle90
#' @description
#' Log-likelihood  
#' @param par parameters
#' @param dist0 Distances 
#' @param dist1 Distances
#' @param h step
#' @return
#' p-value
#' 
.Lneg=function(par,dist0,dist1,h){
    alpha=par[1]
    beta=par[2]
    n0=length(dist0)
    n1=length(dist1) 
    sum1=sum(log(1-alpha*exp(-beta*dist1)))
    sum0=sum(exp(-beta*dist0/(2*beta*h^2+1))/(2*beta*h^2+1))
    -sum1+n1*log(1+alpha*sum0/n0)
}

#' Auxiliary function for diggle90
#' @description
#' Log-likelihood  
#' @param par parameters
#' @param dist0 Distances 
#' @param dist1 Distances
#' @param h step
#' @return
#' p-value
#' 
.Lgrneg=function(par,dist0,dist1,h){
    alpha=par[1]
    beta=par[2]
    n0=length(dist0)
    n1=length(dist1) 
    sum11=sum(exp(-beta*dist1)/(1+alpha*exp(-beta*dist1)))
    sum01=sum(exp(-beta*dist0/(2*beta*h^2+1))/(2*beta*h^2+1))
    w=1+alpha*sum01/n0
    sum12=sum(dist1*exp(-beta*dist1)/(1+alpha*exp(-beta*dist1)))
    sum02=sum((exp(-beta*dist0/(2*beta*h^2+1))/(2*beta*h^2+1))*
              ((dist1+2*h^2+4*beta*h^4)/(2*beta*h^2+1)^2))
    c(-sum11-n1*sum01/w,-(-alpha*sum12+n1*alpha*sum02/w))
}




